#define MOTO_DIR             7
#define MOTO_STEP            6
#define LASER_POWER          5

#define UP_LIMIT             3
#define DOWN_LIMIT           2

unsigned char report_change;
unsigned long prev_report_tm;
unsigned char laser_power_level = 0;
int prev_dir = -1;

void up_limit_proc() {
  report_change = 1;
}

void down_limit_proc() {
  report_change = 1;
}

void setup() {
  Serial.begin(115200);
  
  pinMode(UP_LIMIT, INPUT); attachInterrupt(digitalPinToInterrupt(UP_LIMIT), up_limit_proc, CHANGE); // CHANGE // RISING
  pinMode(DOWN_LIMIT, INPUT); attachInterrupt(digitalPinToInterrupt(DOWN_LIMIT), down_limit_proc, CHANGE); // CHANGE // RISING
  
  pinMode(MOTO_DIR, OUTPUT); digitalWrite(MOTO_DIR, LOW);
  pinMode(MOTO_STEP, OUTPUT); digitalWrite(MOTO_STEP, LOW);
  pinMode(LASER_POWER, OUTPUT); digitalWrite(LASER_POWER, LOW); laser_power_level = 0;

  prev_report_tm = micros();
  report_change = 0;
  prev_dir = -1;
  laser_power_level = 0;
}

int do_steps(int cnt, int dir) {
  while (cnt > 0) {
    if (dir != 0) {
      if (digitalRead(UP_LIMIT) == HIGH) {
        return 1;
      }
    } else {
      if (digitalRead(DOWN_LIMIT) == HIGH) {
        return 2;
      }
    }
    digitalWrite(MOTO_STEP, HIGH);
    delay(1);
    digitalWrite(MOTO_STEP, LOW);
    delay(1);
    cnt--;
  }
  return 0;
}

void do_dir(int dir) {
  if (dir != 0) {
    digitalWrite(MOTO_DIR, LOW);
  } else {
    digitalWrite(MOTO_DIR, HIGH);
  }
}

void make_steps_info(int cnt, int dir) {
  do_dir(dir);
  switch (do_steps(cnt, dir)) {
  case 1:
    Serial.println("BREAK UP");
    break;  
  case 2:
    Serial.println("BREAK DOWN");
    break;    
  case 0:
  default:
    Serial.println("FINISH");
    break;        
  }
}

void loop() {
  unsigned long cur_tm = micros();
  unsigned long delta_tm = cur_tm - prev_report_tm;
  if ((report_change != 0) || (delta_tm > 1000000)) {
    report_change = 0;
    prev_report_tm = cur_tm;
    Serial.print(digitalRead(UP_LIMIT)); Serial.print(", "); Serial.print(digitalRead(DOWN_LIMIT)); 
    Serial.write(0x0D); Serial.write(0x07); Serial.write(0x0A);
  }
  while (Serial.available() > 0) {
    int chr = Serial.read();
    switch (chr) {
    case 'q': case 'Q':
      Serial.println("STEP UP 1");      
      make_steps_info(10, 1);
      break;
    case 'w': case 'W':
      Serial.println("STEP UP 10");
      make_steps_info(100, 1);
      break;
    case 'e': case 'E':
      Serial.println("STEP UP 100");
      make_steps_info(1000, 1);
      break;
    case 'a': case 'A':
      Serial.println("STEP DOWN 1");      
      make_steps_info(10, 0);
      break;
    case 's': case 'S':
      Serial.println("STEP DOWN 10");
      make_steps_info(100, 0);
      break;
    case 'd': case 'D':
      Serial.println("STEP DOWN 100");
      make_steps_info(1000, 0);
      break;
      
    case 'n': case 'N':
      Serial.println("LASER ON");
      digitalWrite(LASER_POWER, HIGH);
      laser_power_level = 255;
      break;
    case 'f': case 'F':
      Serial.println("LASER OFF");
      digitalWrite(LASER_POWER, LOW);
      laser_power_level = 0;
      break;
    case 'o': case 'O':
      Serial.print("LASER MORE: "); Serial.println(laser_power_level);
      if (laser_power_level <= 245) {
        laser_power_level += 10;
      }
      analogWrite(LASER_POWER, laser_power_level);
      break;
    case 'p': case 'P':
      Serial.print("LASER LESS: "); Serial.println(laser_power_level);
      if (laser_power_level >= 10) {
        laser_power_level -= 10;
      }
      analogWrite(LASER_POWER, laser_power_level);
      break;  

    case 0x01:
      make_step_bin(1);
      break;
    case 0x02:
      make_step_bin(0);
      break;
    case 0x03:
      laser_power_level = 255; analogWrite(LASER_POWER, laser_power_level); Serial.write(0x0F);
      break;
    case 0x04:
      laser_power_level = 0; analogWrite(LASER_POWER, laser_power_level); Serial.write(0x0E);
      break;
    case 0x05:
      if (laser_power_level == 0) { laser_power_level = 1; } else if (laser_power_level >= 128) { laser_power_level = 255; } else { laser_power_level *= 2; }
      analogWrite(LASER_POWER, laser_power_level); 
      if (laser_power_level < 255) { Serial.write(0x0B); } else { Serial.write(0x0F); }
      break;
    case 0x06:
      if (laser_power_level > 0) { laser_power_level--; } 
      analogWrite(LASER_POWER, laser_power_level); 
      if (laser_power_level > 0) { Serial.write(0x0C); } else { Serial.write(0x0E); }
      break;
    case 0x07:
      if (laser_power_level < 255) { laser_power_level++; } 
      analogWrite(LASER_POWER, laser_power_level); 
      if (laser_power_level < 255) { Serial.write(0x0B); } else { Serial.write(0x0F); }
      break;
    }
  }
}

void make_step_bin(int dir) {
  unsigned char ans = 0;
  if (prev_dir != dir) {
    prev_dir = dir;
    do_dir(dir);
  }
  switch (do_steps(1, dir)) {
  case 1: ans = 0x11; // BREAK UP
    break;  
  case 2: ans = 0x12; // BREAK DOWN
    break;    
  case 0: default: 
    if (dir != 0) {
      ans = 0x13; // FINISH UP
    } else {
      ans = 0x14; // FINISH DOWN
    }
    break;        
  }
  Serial.write(ans);
}
